<?php

declare(strict_types=1);

namespace Doctrine\Tests\Persistence\Mapping\_files\colocated;

/**
 * The driver should include this file and return its class name
 * from {@see \Doctrine\Persistence\Mapping\Driver\ColocatedMappingDriver::getAllClassNames()} method.
 */
class Entity
{
}
